/*
 *Avaliacao 1: 1s2012
 *
 * Neste codigo mostramos as duas curvas: uma Bezier (em branco) e outra 
 *       BSpline (em azul berilo)
 *
 * Observe que foi aplicada um deslocamento em cada curva. A posicao 
 * relativa entre os pontos de controle e a curva  preservada quando
 * comparada com o estado antes do deslocamento?
 *
 * Possiveis interacoes:
 *    rotacao em torno de x: 'x'; em torno de y: 'y'; em torno de z: 'z'
 *    reset 'r'
 *
 * Questao 3: 
 *     (b) Altere a posicao de pontos de controle e observe a alteracao
 *         nas curvas
 *     (c) Desenhe o poligono "convexo" no qual a curva fica contida
 *     (d) Altere os ns de forma que os extremos da curva BSpline conicidam
 *         com os extremos do poligono de controle
 *
 */
#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>

GLfloat ctrlpoints[8][3] = {
        { -8.0, -4.0, 0.0}, { -6.0, 4.0, 0.0}, 
        {-4.0, -4.0, 0.0}, {-2.0, 4.0, 0.0},
	{ 2.0, -4.0, 0.0}, { 4.0, 4.0, 0.0}, 
        {6.0, -4.0, 0.0}, {8.0, 4.0, 0.0}};

GLfloat knots[13] = {0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0,11.0};

GLUnurbsObj *theNurb;

void nurbsError(GLenum errorCode)
{
   const GLubyte *estring;

   estring = gluErrorString(errorCode);
   fprintf (stderr, "Nurbs Error: %s\n", estring);
   exit (0);
}

void init(void)
{
   /* definir a cor em preto (0,0,0) como cor de "CLEAR" */
   glClearColor (0.0, 0.0, 0.0, 0.0);
   /* setar o tipo de tonalizacao. Ha duas alternativas
      GL_FLAT (constante) e GL_SMOOTH (interpolacao linear
      entre os vertices) */
   glShadeModel (GL_FLAT);
   /* setar o modo de renderizacao das faces. Ha tres alternativas:
      GL_POINT (somente vertices), GL_LINE (somente linhas) e GL_FILL
      (faces preenchidas) */
   glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
   /* setar a convencao de orientacao. Ha duas alternativas: GL_CCW (anti-horario)      e GL_CW (horario) */
   glFrontFace (GL_CCW);
   /* Habilitar face culling */
   glEnable (GL_CULL_FACE);
   /* definir as faces a serem discardadas. Ha tres alternativas: GL_FRONT,
      GL_BACK e GL_FRONT_AND_BACK */
   glCullFace (GL_FRONT);

   /* definir e habilitar funcao de computo das amostras */
   glMap1f(GL_MAP1_VERTEX_3, 0.0, 1.0, 3, 8, &ctrlpoints[0][0]);
   glEnable(GL_MAP1_VERTEX_3);

   theNurb = gluNewNurbsRenderer();
   gluNurbsProperty(theNurb, GLU_SAMPLING_TOLERANCE, 25.0);
   gluNurbsCallback(theNurb, GLU_ERROR, 
                    (GLvoid (*)()) nurbsError);
}

void display(void)
{
   int i;

   /* "CLEAR" o display */
   glClear (GL_COLOR_BUFFER_BIT);

   /* BEZIER */
   /* carregar a matriz de identidade na pilha de matriz de transformacao */
   glLoadIdentity();

   glPushMatrix();
   /* deslocar para a parte superior da janela */
   glTranslatef (0., 8., 0.);

   /* desenhar uma tira de amostras */
   glBegin(GL_LINE_STRIP);
      for (i = 0; i <= 30; i++) 
         glEvalCoord1f((GLfloat) i/30.0);
   glEnd();
   
   /* desenhar o poligno de controle */
   glPointSize(5.0);
   glColor3f(1.0, 1.0, 0.0);
   glBegin(GL_POINTS);
      for (i = 0; i < 8; i++) 
         glVertex3fv(&ctrlpoints[i][0]);
   glEnd();
   glPopMatrix();


   /* BSPLINE */
   glPushMatrix();

   /* deslocar para a parte inferior da janela */
   glTranslatef (0., -8., 0.);

   /* desenhar Bspline de ordem 4 */
   glColor3f(0.0, 1.0, 1.0);
   gluBeginCurve(theNurb);
   gluNurbsCurve(theNurb, 
                   11, knots, 
                   3, &ctrlpoints[0][0], 
                   4, GL_MAP1_VERTEX_3);
   gluEndCurve(theNurb);

   /* desenhar o poligno de controle */
   glPointSize(5.0);
   glColor3f(1.0, 1.0, 0.0);
   glBegin(GL_POINTS);
      for (i = 0; i < 8; i++) 
         glVertex3fv(&ctrlpoints[i][0]);
   glEnd();
   glPopMatrix();

   /* forcar a execucao dos comandos enviados a OpenGL */
   glFlush ();
}

void reshape (int w, int h)
{
   /* definir o tamanha de janela no display, em pixels */
   //glViewport (0, 0, (GLsizei) w, (GLsizei) h);
   glViewport (0, 0, 500, 500);
   /* chavear para a pilha de transformacao de projecao */
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
   // Volume de interesse: (-10,10,-10,10,-15.0,15.0)
   glOrtho (-10., 10., -20., 20., -15.0, 15.0);
   /* chavear para a pilha de transformacao de modelo */
   glMatrixMode (GL_MODELVIEW);
}

/* ARGSUSED1 */
void keyboard(unsigned char key, int x, int y)
{
   switch (key) {
      case 27:
         exit(0);
         return;
   }
   /* forar um evento para ativar o redesenho */
   glutPostRedisplay();
}

int main(int argc, char** argv)
{
   /* estabelecimento de comunicacao com o sistema de janela */
   glutInit(&argc, argv);
   /* setar o " visual" do display */
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
   /* definir a janela */
   glutInitWindowSize (500, 500);
   glutInitWindowPosition (100, 100);
   glutCreateWindow (argv[0]);
   /* inicializar openGL */
   init ();
   /* registrar os tratadores de eventos */
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   /* entre no modo de espera */
   glutMainLoop();
   return 0;
}
