/*
 *Avaliacao 1: 1s2012
 *
 * Neste codigo mostramos a vista do cubo segundo os parametros
 * dados na questao 5. Foram utilizadas as funcoes gluLookAt
 * e glFrustum. 
 *
 * Observe que os parametros de glFrustum para implementar o 
 *     volume de interesse sao ligeiramente diferentes. Explique.
 *  // Volume de interesse: (-2,2,-1,4,0.0,5.0)
 *  glFrustum (-2., 2., -1., 4., 1.0, 6.0);
 *
 * Possiveis interacoes:
 *    rotacao em torno de x: 'x'; em torno de y: 'y'; em torno de z: 'z'
 *    reset 'r'
 *
 * Questao 5: Altere o cdigo para gerar
 * (c) uma projeo Cabinet do cubo
 * (d) gerar a mesma vista perspectiva sem utilir gluLookAt (Dica: se precisar,
 *                   utilize glMultMatrix())
 *
 */
#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>

// Definica de um cubo unitario, centrado na origem 
#define v .577350269
#define m 0.5

static GLfloat vdata[8][3] = {
   {-m, -m, -m}, {m, -m, -m}, {m, m, -m}, {-m, m, -m},
   {-m, -m, m}, {m, -m, m}, {m, m, m}, {-m, m, m}
   };

static GLfloat ndata[8][3] = {
   {-v, -v, -v}, {v, -v, -v}, {v,v,-v}, {-v, v, -v},
   {-v, -v, v}, {v, -v, v}, {v,v,v}, {-v, v, v}
   };

static GLuint vindices[6][4] = {
   {0,3,2,1}, {2,3,7,6}, {0,4,7,3},
   {1,2,6,5}, {4,5,6,7}, {0,1,5,4}
   };

GLfloat spin_x=0.0, spin_y=0.0, spin_z=0.0;

void init(void)
{
   /* definir a cor em preto (0,0,0) como cor de "CLEAR" */
   glClearColor (0.0, 0.0, 0.0, 0.0);
   /* setar o tipo de tonalizacao. Ha duas alternativas
      GL_FLAT (constante) e GL_SMOOTH (interpolacao linear
      entre os vertices) */
   glShadeModel (GL_FLAT);
   /* setar o modo de renderizacao das faces. Ha tres alternativas:
      GL_POINT (somente vertices), GL_LINE (somente linhas) e GL_FILL
      (faces preenchidas) */
   glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
   /* setar a convencao de orientacao. Ha duas alternativas: GL_CCW (anti-horario)      e GL_CW (horario) */
   glFrontFace (GL_CCW);
   /* Habilitar face culling */
   glEnable (GL_CULL_FACE);
   /* definir as faces a serem discardadas. Ha tres alternativas: GL_FRONT,
      GL_BACK e GL_FRONT_AND_BACK */
   glCullFace (GL_FRONT);
}

void display(void)
{
   int i;

   /* "CLEAR" o display */
   glClear (GL_COLOR_BUFFER_BIT);
   /* setar a cor de desenho em branco (1,1,1) */
   glColor3f (1.0, 1.0, 1.0);
   /* carregar a matriz de identidade na pilha de matriz de transformacao */
   glLoadIdentity();

   /* 
    * Na especificacao:
    *    Observador: (1.,1.,1.,1.)
    *    Centro de interesse: (0.,0.,0.,1.)
    *    VUP: (0.,1.,0.,0.)
    * Camra da OpenGL: esta na origem e VUP//y, portanto precisamos
    *         transformar a cena para que as coordenadas
    *         sejam em relacao a este referencial antes
    *         da projecao
    *
    */
   gluLookAt (1.,1.,1.,0.,0.,0.,0.,1.,0.);

   glRotatef(spin_x, 1.0 , 0., 0.);
   glRotatef(spin_y, 0.  , 1., 0.);
   glRotatef(spin_z, 0., 0., 1.0);

   /* desenhar o cubo */
   for (i=0; i<6; i++) {
     glBegin(GL_POLYGON);
     glNormal3fv(&ndata[vindices[i][0]][0]);
     glVertex3fv(&vdata[vindices[i][0]][0]);
     glNormal3fv(&ndata[vindices[i][1]][0]);
     glVertex3fv(&vdata[vindices[i][1]][0]);
     glNormal3fv(&ndata[vindices[i][2]][0]);
     glVertex3fv(&vdata[vindices[i][2]][0]);
     glNormal3fv(&ndata[vindices[i][3]][0]);
     glVertex3fv(&vdata[vindices[i][3]][0]);
     glEnd();
   }

   /* forcar a execucao dos comandos enviados a OpenGL */
   glFlush ();
}

void reshape (int w, int h)
{
   /* definir o tamanha de janela no display, em pixels */
   //glViewport (0, 0, (GLsizei) w, (GLsizei) h);
   glViewport (0, 0, 500, 500);
   /* chavear para a pilha de transformacao de projecao */
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
   // Volume de interesse: (-2,2,-1,4,0.0,5.0)
   glFrustum (-2., 2., -1., 4., 1.0, 6.0);
   /* chavear para a pilha de transformacao de modelo */
   glMatrixMode (GL_MODELVIEW);
}

/* ARGSUSED1 */
void keyboard(unsigned char key, int x, int y)
{
   switch (key) {
      case 'x':
        /* increment +10.0 no ngulo de rotao em torno do eixo x */
	spin_x += 10.0;
        break;
      case 'y':
        /* incrementa +10.0 no ngulo de rotao em torno do eixo y */
	spin_y += 10.0;
        break;
      case 'z':
        /* increment +10.0 no ngulo de rotao em torno do eixo z */
	spin_z += 10.0;
        break;
      case 'r':
        /* reset */
	spin_x = spin_y = spin_z = 0.0;
        break;
      case 27:
         exit(0);
         return;
   }
   /* forar um evento para ativar o redesenho */
   glutPostRedisplay();
}

int main(int argc, char** argv)
{
   /* estabelecimento de comunicacao com o sistema de janela */
   glutInit(&argc, argv);
   /* setar o " visual" do display */
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
   /* definir a janela */
   glutInitWindowSize (500, 500);
   glutInitWindowPosition (100, 100);
   glutCreateWindow (argv[0]);
   /* inicializar openGL */
   init ();
   /* registrar os tratadores de eventos */
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   /* entre no modo de espera */
   glutMainLoop();
   return 0;
}
